Module keyboard

Module keyboard 

Structs

KeyInput
Represents a key definition that can be dispatched to the browser.
KeyModifiers
Bitmask wrapper that tracks active modifiers.
Keyboard
High-level helper for dispatching keyboard events through the CDP Input domain.

Enums

KeyboardModifier
Supported keyboard modifiers.