TracingController

Struct TracingController 

pub struct TracingController { /* private fields */ }
High-level controller for Chrome tracing on a single Page.

Examples

let tracing = page.tracing();
tracing.start(Some(TracingStartOptions::default())).await?;
// ... perform actions to record ...
let result = tracing.stop(None).await?;
assert!(!result.data.is_empty());

Implementations

Source

impl TracingController

pub async fn is_recording(&self) -> bool

Returns true if tracing has been started and not yet fully stopped.

pub async fn categories(&self) -> Result<Vec<String>>

Queries the browser for supported tracing categories.

pub async fn start(&self, options: Option<TracingStartOptions>) -> Result<()>

Starts tracing with the provided options (or defaults when None).

pub async fn stop( &self, save_path: Option<PathBuf>, ) -> Result<TracingStopResult>

Stops tracing and optionally persists the trace to disk.