pub struct TracingStartOptions {
pub categories: Option<Vec<String>>,
pub screenshots: bool,
pub record_mode: TraceConfigRecordMode,
pub buffer_usage_reporting_interval_ms: Option<f64>,
pub stream_format: StreamFormat,
pub stream_compression: StreamCompression,
pub tracing_backend: Option<TracingBackend>,
}Expand description
Configuration used when starting a tracing session.
Fields
categories: Option<Vec<String>>Categories to trace; defaults to a curated set that captures page activity.
screenshots: boolWhether to capture screenshots in addition to trace events.
record_mode: TraceConfigRecordModeTrace record mode (see TraceConfig.record_mode).
buffer_usage_reporting_interval_ms: Option<f64>Optional buffer usage reporting interval in milliseconds.
stream_format: StreamFormatRequested stream format for trace data.
stream_compression: StreamCompressionRequested compression format for the streamed trace.
tracing_backend: Option<TracingBackend>Optional tracing backend to use.