pub struct TracingController { /* private fields */ }Expand description
High-level controller for Chrome tracing on a single Page.
Examples
let tracing = page.tracing();
tracing.start(Some(TracingStartOptions::default())).await?;
// ... perform actions to record ...
let result = tracing.stop(None).await?;
assert!(!result.data.is_empty());Implementations
Sourceimpl TracingController
impl TracingController
Sourcepub async fn is_recording(&self) -> bool
pub async fn is_recording(&self) -> bool
Returns true if tracing has been started and not yet fully stopped.
Sourcepub async fn categories(&self) -> Result<Vec<String>>
pub async fn categories(&self) -> Result<Vec<String>>
Queries the browser for supported tracing categories.